#include "AutoFunctions.h"
AutoFunctions::AutoFunctions(Kicker* theKicker, ThunderDrive* thunderDrive, ThunderCam* camera) {	
	powerLevel = 0;
	myKicker = theKicker;
	tDrive = thunderDrive;
	tCam = camera;
	pidMaxSpeed = 0;
	kickerBroken = false;

	currentBall = 1;
	distanceToCurrentBall = 0;
	angleToCurrentBall = 0;
	angleToGoalCurrentBall = 0;
	closerToGoalDistanceCurrentBall = 0;
	backwardDistance = 0;
	gameZone = 0;
	delayTime = 0;
	begin_time = 0;
	runWithCamera = false;
	currentAngleToGoal = 0;
	
	maxAngle = 0;
	minAngle = 0;
	
	distanceToBall1 = 0;
	closerToGoalDistanceBall1 = 0;
	backwardDistanceBall1 = 0;
	angleToBall1 = 0;
	angleToGoalBall1 = 0;
	
	distanceToBall2 = 0;
	closerToGoalDistanceBall2 = 0;
	backwardDistanceBall2 = 0;
	angleToBall2 = 0;
	angleToGoalBall2 = 0;
	
	distanceToBall3 = 0;
	closerToGoalDistanceBall3 = 0;
	backwardDistanceBall3 = 0;
	angleToBall3 = 0;
	angleToGoalBall3 = 0;
	
	camCanSeeBalls = false;
	reckoningCurrentStep = AutoFunctions::rStartDelay;
}

AutoFunctions::~AutoFunctions() {
    
}

void AutoFunctions::runAuto() {
	tDrive->Process();
	//tCam->Process();
	//printf("State: %d\n", reckoningCurrentStep);
	switch (reckoningCurrentStep) {
		case AutoFunctions::rStartDelay: 
			printf("Current Ball is %i\n", currentBall);
			begin_time = GetTime();
			printf("Start Delay STate for %f seconds\n", (double)delayTime);
			reckoningCurrentStep = AutoFunctions::rCheckDelay;
			break;
			
		case AutoFunctions::rCheckDelay:
			double curTime = GetTime();
			if ((curTime >= (begin_time + (double)delayTime)) && myKicker->GetPreMatchDone())
			{
				printf("Delay State Complete\n");
				// resetting the gyro to 0 in case it is too close to 300 degress
				// thunderdrive doesn't try to move out of range
				tDrive->resetGyro();
				reckoningCurrentStep = AutoFunctions::rBrushOnState;
			}
			break;
			
		case AutoFunctions::rBrushOnState: 
			myKicker->TurnBeaterBrush(1);
			printf("Beater Brush STate\n");
			reckoningCurrentStep = AutoFunctions::rSetKickingForceState;
			break;
		
		case AutoFunctions::rSetKickingForceState:
			if (!kickerBroken) {
				myKicker->SetPowerDesired(powerLevel);
				printf("Kicking Force STate\n");
			}
			else {
				printf("Kicker is broken - skipping step\n");
			}
			reckoningCurrentStep = AutoFunctions::rPreAngleToBallState;
			break;
		
		case AutoFunctions::rPreAngleToBallState:
				tDrive->AimAngle(angleToCurrentBall);
				updateAngle(angleToCurrentBall);
				reckoningCurrentStep = AutoFunctions::rAngleToBallState;
				printf("Angle to ball STate given angle is %f\n", (double) angleToCurrentBall);
			break;
		
		case AutoFunctions::rAngleToBallState:
			if (tDrive->AutoMotionDone()) {
				tDrive->Stop(); //just to be safe
				printf("Done angleing to ball STate\n");
				reckoningCurrentStep = AutoFunctions::rPreDriveToBallState;
			}
			else {
				//printf("Auto motion not done %f\n", (double)tDrive->getRobotAngle());
				//printf("Angle to Current Ball is %f", angleToCurrentBall);
				checkCourse(); //make sure we are still on course
			}
			break;
		
		case AutoFunctions::rPreDriveToBallState: 
			pidMaxSpeed = GetSpeed(distanceToCurrentBall);
			tDrive->DriveDistance(distanceToCurrentBall, pidMaxSpeed);
			updateAngle(0);
			printf("Drive To Ball STate given distance is %f\n", (double) distanceToCurrentBall);
			reckoningCurrentStep = AutoFunctions::rDriveUntilBallState;
			break;
		
		case AutoFunctions::rDriveUntilBallState: 
			if (tDrive->AutoMotionDone()/* && myKicker->BallAvailable()*/) {
				// stops if we have ball before distance done 
				tDrive->Stop();
				printf("Done driving to ball STate\n");
				reckoningCurrentStep = AutoFunctions::rPreAngleToGoalState;
			}
			else {
				checkCourse(); // we are no longer driving straight
			}
			break;
		
		case AutoFunctions::rPreAngleToGoalState: 
			tDrive->AimAngle(angleToGoalCurrentBall);
			updateAngle(angleToGoalCurrentBall);
			printf("Aiming to Goal STate\n");
			reckoningCurrentStep = AutoFunctions::rAngleToGoalState;
			break;
		
		case AutoFunctions::rAngleToGoalState: 
			if (tDrive->AutoMotionDone()) {
				tDrive->Stop();
				printf("Done angling to goal STate\n");
				reckoningCurrentStep = AutoFunctions::rPreDriveToGoalState;
			}
			else {
				checkCourse(); //we are no longer turning correctly
			}
			break;
		
		case AutoFunctions::rPreDriveToGoalState:
			tDrive->DriveDistance(closerToGoalDistanceCurrentBall, GetSpeed(closerToGoalDistanceCurrentBall));
			updateAngle(0);
			printf("Drive to goal STate\n");
			reckoningCurrentStep = AutoFunctions::rDriveToGoalState;
			break;
		
		case AutoFunctions::rDriveToGoalState: 
			if (tDrive->AutoMotionDone()) {
				tDrive->Stop();//just to be safe - spazzzbot
				printf("Done driving to goal STate\n");
				reckoningCurrentStep = AutoFunctions::rPreAngleToGoalCamState;
			}
			else {
				checkCourse(); // we are no longer driving straight
			}
			break;
		
		case AutoFunctions::rPreAngleToGoalCamState: 
			if (runWithCamera && (tCam != NULL)) {
				printf("Running with camera\n");
				tCam->setTargetType(true);
				if (tCam->targetFound()) {
					float targetAngle = tCam->getAngleToTarget();
					if ((targetAngle <= CAM_MAX_TARGET_ANGLE) && (targetAngle >= -CAM_MAX_TARGET_ANGLE))
					{
						tDrive->AimAngle(targetAngle);
						printf("Camera aiming to target\n");
						updateAngle(targetAngle);
						reckoningCurrentStep = AutoFunctions::rAngleToGoalCamState;
					}
				}
			}
			else {
				printf("Not using camera - skipping step\n");
				reckoningCurrentStep = AutoFunctions::rKickBallState;
			}
			break;
		
		case AutoFunctions::rAngleToGoalCamState: 
			if (tDrive->AutoMotionDone()) {
				tDrive->Stop();
				updateAngle(0);
				printf("Done angling to goal with camera\n");
				reckoningCurrentStep = AutoFunctions::rKickBallState;
			}
			else {
				checkCourse(); // if we are spinning uncontrollably waiting for 
			}
			break;
		
		case AutoFunctions::rKickBallState:
			if (kickerBroken) {
				reckoningCurrentStep = AutoFunctions::rTiltCamState;
				break;
			}
			if (checkCourse()) {	
				if (myKicker->Fire()) {
					printf("Kicker has fired\n");
					printf("Current Ball is %i\n", currentBall);
					if (currentBall == 1) {
						printf("Current Ball is 1, Zone is %i\n", gameZone);
						if (gameZone == 2 || gameZone == 3) {
							distanceToCurrentBall = distanceToBall2;
							angleToCurrentBall = angleToBall2;
							angleToGoalCurrentBall = angleToGoalBall2;
							closerToGoalDistanceCurrentBall = closerToGoalDistanceBall2;
							printf("Moving to next ball\n");
							currentBall++;
							reckoningCurrentStep = AutoFunctions::rPreAngleToBallState;
							break;
						}
					}
					else if (currentBall == 2) {
						if (gameZone == 3) {
							distanceToCurrentBall = distanceToBall3;
							angleToCurrentBall = angleToBall3;
							angleToGoalCurrentBall = angleToGoalBall3;
							closerToGoalDistanceCurrentBall = closerToGoalDistanceBall3;
							printf("Moving to next ball STate\n");
							currentBall++;
							reckoningCurrentStep = AutoFunctions::rPreAngleToBallState;
							break;
						}
					}
					reckoningCurrentStep = AutoFunctions::rPreBackingUpState;
				}
			}
			break;
		
		case AutoFunctions::rPreBackingUpState:
		    pidMaxSpeed = GetSpeed(backwardDistance);
			tDrive->DriveDistance(-backwardDistance, pidMaxSpeed);
			printf("Backing up STate\n");
			updateAngle(0);
			reckoningCurrentStep = AutoFunctions::rBackingUpState;
			break;
		
		case AutoFunctions::rBackingUpState: 
			if (tDrive->AutoMotionDone()) {
				tDrive->Stop();//just to be safe
				printf("Done backing up STate\n");
				reckoningCurrentStep = AutoFunctions::rTiltCamState;
			}
			else {
				checkCourse(); // we are no longer backing up straight
			}
			break;
		
		case AutoFunctions::rTiltCamState: 
			tCam->setServoValue(CAM_DOWN_ANGLE);
			printf("Tilting Camera STate\n");
			reckoningCurrentStep = AutoFunctions::rCompleted;
			printf("MODE IS COMPLETED!!!!!!!\n");
			break;
		
		case AutoFunctions::rCompleted:
			break;
		default:
			printf("ERROR: Unhandled Mode Number Called\n");			
			break;
	
	}
}

void AutoFunctions::BlockTunnel() {
	tDrive->AimAngle(62.614418547882836222267434458515);
	if(tDrive->AutoMotionDone())
	tDrive->DriveDistance(96, 1);
}

void AutoFunctions::setModeParameters(float desiredPowerLevel,
			float distanceBackward,
			int zone,
			bool cameraMode) {
	gameZone = zone;
	powerLevel = desiredPowerLevel;
	backwardDistance = distanceBackward;
	reckoningCurrentStep = AutoFunctions::rStartDelay;
	runWithCamera = cameraMode;
}
void AutoFunctions::setBallOne (float angleToBall,
			float distance,
			float angleToGoal,
			float driveToGoal) {
    distanceToBall1 = distance;
	closerToGoalDistanceBall1 = driveToGoal;
	backwardDistanceBall1 = angleToGoal;
	angleToBall1 = angleToBall;
	angleToGoalBall1 = angleToGoal;
	currentBall = 1;
	
	distanceToCurrentBall = distanceToBall1;
	angleToCurrentBall = angleToBall1;
	angleToGoalCurrentBall = angleToGoalBall1;
	closerToGoalDistanceCurrentBall = closerToGoalDistanceBall1;
}
	
void AutoFunctions::setBallTwo (float angleToBall,
			float distance,
			float angleToGoal,
			float driveToGoal) {
	distanceToBall2 = distance;
	closerToGoalDistanceBall2 = driveToGoal;
	backwardDistanceBall2 = angleToGoal;
	angleToBall2 = angleToBall;
	angleToGoalBall2 = angleToGoal;
}
	
void AutoFunctions::setBallThree (float angleToBall,
			float distance,
			float angleToGoal,
			float driveToGoal)  {
	distanceToBall3 = distance;
	closerToGoalDistanceBall3 = driveToGoal;
	backwardDistanceBall3 = angleToGoal;
	angleToBall2 = angleToBall;
	angleToGoalBall2 = angleToGoal;
}
void AutoFunctions::endAuto() {
	reckoningCurrentStep = AutoFunctions::rCompleted;
	tDrive->Stop();
	tCam->setServoValue(CAM_DOWN_ANGLE);
}
void AutoFunctions::setDelay(float time) {
	delayTime = time;
}
void AutoFunctions::updateAngle(float desiredAngle) {
	float currentAngle = tDrive->getRobotAngle();
	printf("Updating angle tolerance based on cur %f + desired %f\n", (double)currentAngle, (double)desiredAngle);

	float newAngle = currentAngle + desiredAngle;
	if (newAngle >= currentAngle)
	{
		minAngle = currentAngle - ERROR_MARGIN;
		maxAngle = newAngle + ERROR_MARGIN;
	}
	else
	{
		minAngle = newAngle - ERROR_MARGIN;
		maxAngle = currentAngle + ERROR_MARGIN;
	}
}
bool AutoFunctions::checkCourse() {
	float currentAngle = tDrive->getRobotAngle();
	if ((currentAngle >= minAngle) && (currentAngle <= maxAngle)) {
		return true;
	}
	else {
		endAuto();
		printf("Ended because %f outside %f -> %f\n", (double)currentAngle, (double)minAngle, (double)maxAngle);
		return false;
	}
}
float AutoFunctions::GetSpeed(float distance) {
	if (distance <= 12.0 && distance >= 0) {
		return 0.3f;
	}
	else if (distance > 12.0 && distance >= 0){
		return 0.5f;
	}
	else if (distance < 0) {
		return 0.4f;
	}
	return 0.0f;
	
}
void AutoFunctions::resetState() {
	reckoningCurrentStep = AutoFunctions::rStartDelay;
	currentBall = 1;
}
void AutoFunctions::setKickerBroken(bool broken) {
	kickerBroken = broken;
}

