#ifndef AUTOFUNCTIONS_H_
#define AUTOFUNCTIONS_H_
#include "Kicker.h"
#include "ThunderDrive.h"
#include "ThunderCam.h"
#include "timers.h"
#include "Controls.h"
#define POWER_LEVEL_NEAR 5.0f
#define POWER_LEVEL_MIDDLE 10.0f
#define POWER_LEVEL_FAR 15.0f
#define DISTANCE_BALL_NEAR 5.0f
#define DISTANCE_BALL_MIDDLE 5.0f
#define DISTANCE_BALL_FAR 5.0f
#define DISTANCE_CLOSER_BALL 5.0f
#define DISTANCE_CLOSER_GOAL 5.0f
#define DISTANCE_BACKWARD 5.0f
#define CAM_DOWN_ANGLE -30
#define ERROR_MARGIN 25 //number of degrees of error to allow when driving
#define CAM_MAX_TARGET_ANGLE 15 // number of degress off center the target can be for it to be valid 


#define ANGLE_BALL_ONE 90
#define ANGLE_BALL_TWO 90
#define ANGLE_BALL_THREE 90

#define DISTANCE_BALL_ONE 5
#define DISTANCE_BALL_TWO 5
#define DISTANCE_BALL_THREE 5

#define ANGLE_GOAL_BALL_ONE 90
#define ANGLE_GOAL_BALL_TWO 90
#define ANGLE_GOAL_BALL_THREE 90

#define GOAL_DISTANCE_BALL_ONE 5
#define GOAL_DISTANCE_BALL_TWO 5
#define GOAL_DISTANCE_BALL_THREE 5


class AutoFunctions {
public:
	//constructor
	AutoFunctions(Kicker* theKicker, ThunderDrive* thunderDrive, ThunderCam* camera);
	~AutoFunctions();
	// Autonomous function based off using dead reckoning 
	void runAuto();
	// stops everything except beater brush
	void endAuto();
	// sets the delay time for the start
	void setDelay(float time);
	// updates what the max and min angles will be depending on what your desired angle is
	void updateAngle(float desiredAngle);
	// checks if we are off course
	bool checkCourse();
	//turn towards tunnel, drive straight to it
	void BlockTunnel();
/*
	// Autonomous functions based on using the camera to aim
	void optionCameraGuided();

	// Autonomous function that you use if the camera can track balls
	void optionCameraSeesBalls();
	*/
	// sets the parameters that the mode will use to run
	void setModeParameters(float desiredPowerLevel,
			float distanceBackward,
			int zone,
			bool cameraMode);
	
	void setBallOne (float angleToBall,
			float distance,
			float angleToGoal,
			float driveToGoal);
	
	void setBallTwo (float angleToBall,
				float distance,
				float angleToGoal,
				float driveToGoal);
	
	void setBallThree (float angleToBall,
				float distance,
				float angleToGoal,
				float driveToGoal);
	
	// gets the max speed based on the distance we are going to travel
	float GetSpeed (float distance);
	
	// resets the state in the sate machine to the first one: rStartDelay
	void resetState();
	
	// tells autonomous that the kicker power level sensor is broken
	// autonomous will run but will not be doing anything with the kicker
	void setKickerBroken(bool broken);
private:
	Kicker* myKicker;
	ThunderDrive* tDrive;
	ThunderCam* tCam;
	float powerLevel;
	int currentBall;
	bool runWithCamera;
	float delayTime;
	double begin_time;
	float currentAngleToGoal;
	float pidMaxSpeed;
	bool kickerBroken;
	//to make sure we are still driving on course.
	float minAngle;					// min angle we need to stay within
	float maxAngle;					// max angle we need to stay within
	
	float backwardDistance;
	float distanceToCurrentBall;
	float angleToCurrentBall;
	float angleToGoalCurrentBall;
	float closerToGoalDistanceCurrentBall;
	float distanceToBall1;
	float closerToGoalDistanceBall1;
	float backwardDistanceBall1;
	float angleToBall1;
	float angleToGoalBall1;
	
	float distanceToBall2;
	float closerToGoalDistanceBall2;
	float backwardDistanceBall2;
	float angleToBall2;
	float angleToGoalBall2;
	
	float distanceToBall3;
	float closerToGoalDistanceBall3;
	float backwardDistanceBall3;
	float angleToBall3;
	float angleToGoalBall3;
	
	bool camCanSeeBalls;
	int gameZone;
	enum reckoningStep {rStartDelay, 
		rCheckDelay, 
		rBrushOnState, 
		rSetKickingForceState,
		rPreAngleToBallState,
		rAngleToBallState, 
		rPreDriveToBallState,
		rDriveUntilBallState,
		rPreAngleToGoalCamState,
		rAngleToGoalCamState,
		rPreAngleToGoalState, 
		rAngleToGoalState, 
		rPreDriveToGoalState,
		rDriveToGoalState, 
		rKickBallState,
		rPreBackingUpState,
		rBackingUpState,
		rTiltCamState,
		rCompleted
		};
	
	
	
	
	enum reckoningStep reckoningCurrentStep;
}; 


#endif

