#include "stdio.h"
#include "Controls.h"
#include "Kicker.h"
#include "ThunderDrive.h"
#include "DSMap.h"
#include "Timer.h"
#define USING_GLOW 1

//bool cake = true xor false

Controls::Controls(Joystick *stickLeft, Joystick *stickRight, ThunderDrive *drive, Kicker *kicker, ThunderCam *camera, NewHanger *hanger)
: 	driverStation(DriverStation::GetInstance()),
	driverStationio(driverStation->GetEnhancedIO()),
	driverStationLCD(DriverStationLCD::GetInstance()),
	nextDebugTime(0),
	nextEIOCheckTime(-1.0)
{ 
    leftStick = stickLeft;
    rightStick = stickRight;
    this->drive = drive;
    this->kicker = kicker;
    this->tCamera = camera;
    this->tHanger = hanger;
#if USING_GLOW
    tGlow = new ThunderGlow();
#endif
    checkForEnhancedIO();
}
Controls::~Controls(){
#if USING_GLOW
	delete tGlow;
#endif
}

void Controls::checkForEnhancedIO(void) {
    if (hasEnhancedIO || GetTime() < nextEIOCheckTime)
        return;
    
    //checks to see if the control board has enhanced IO plugged in
    //ALWAYS CHECK IF hasEnhancedIO IS TRUE WHENEVER CALLING AN ENHANCED IO FUNCTION
    //FAILURE TO DO THIS WILL RESULT IN ERRORS AND POSSIBLY A DR. OCTOGONAPUS ATTACK
    if (driverStationio.GetDigitalConfig(1) == DriverStationEnhancedIO::kUnknown)
        hasEnhancedIO = false;
    else {
        hasEnhancedIO = true;
        configureDriversStation();
    }
    nextEIOCheckTime = GetTime() + 2;
}

void Controls::configureDriversStation(void) {
    DriverStationEnhancedIO::tDigitalConfig digIOTable[16];
    // All pull ups except .....
    for (int i = 0; i < 16; ++i)
        digIOTable[i] = DriverStationEnhancedIO::kInputPullUp;
    // ... the outputs
    digIOTable[DS_DIGIO_LED_BALL_READY-1] = DriverStationEnhancedIO::kOutput;
    digIOTable[DS_DIGIO_LED_SPARE_GREEN-1] = DriverStationEnhancedIO::kOutput;
    for (int i = 0; i < 16; ++i)
        driverStationio.SetDigitalConfig(1 + i, digIOTable[i]);
}

void Controls::checkDisables() {
	if (hasEnhancedIO){    //Sets the beater brush direction  ***NEEDS TESTING***
		if(driverStationio.GetDigital(DS_DIGIO_BALL_RDY_WORKING)){
			kicker->SetBallSensorBroken(true);
		}
		else{
			kicker->SetBallSensorBroken(false);
		}

		if (!isKickerWorking()) {
			kicker->SetPowerLevelSensorBroken(true);
		}
		else{
			kicker->SetPowerLevelSensorBroken(false);
		}
		
		if(driverStationio.GetDigital(DS_DIGIO_ARM_WORKING)){
			tHanger->SetHangerSensorBroken(true);
		}
		else{
			tHanger->SetHangerSensorBroken(false);
		}
	}
}

void Controls::runControls() {
	// Drivetrain
	float rightPower = computePower(rightStick);  //Gets the values of the left and right sticks
	float leftPower = computePower(leftStick);
	
	if (rightStick->GetRawButton(DS_JOYBTN_PIVOT)){  //If the right pivot button is pushed
		drive->AimRate(rightStick->GetX());  //use the right x-value to pivot
	}
	else if (leftStick->GetRawButton(DS_JOYBTN_PIVOT)){  //if the left pivot button is pushed
		drive->AimRate(leftStick->GetX());  //use the left x-value to pivot
	}
	else{
		drive->SetMotorPower(leftPower, rightPower);  //else, drive motors normally.
	}
	
	if (hasEnhancedIO) {
			if (!driverStationio.GetDigital(DS_DIGIO_KICK)) {   //If fire button is pushed
				kicker->Fire();  //Fire
			}
	}
	// End Drivetraing
	// Hanger
	float anavolts = driverStation->GetAnalogIn(DS_ANAIN_AUTO_DELAY);
	tHanger->SetManualHangerPower(anavolts / 3.3);	
	if (hasEnhancedIO) {
		// Hanger up/down control
		if (!driverStationio.GetDigital(DS_DIGIO_ARM_UP)) {
			tHanger->HangerUpDown(1);
		}
		else if (!driverStationio.GetDigital(DS_DIGIO_ARM_DOWN)) {
			tHanger->HangerUpDown(-1);
		}
		else {
			tHanger->HangerUpDown(0);
		}
		
		// Hanger left/right
		if (!driverStationio.GetDigital(DS_DIGIO_ARM_LEFT)) {
			tHanger->HangerLeftRight(-1);
		}
		else if (!driverStationio.GetDigital(DS_DIGIO_ARM_RIGHT)) {
			tHanger->HangerLeftRight(1);
		}
		else {
			tHanger->HangerLeftRight(0);
		}
		
		tHanger->HangerLock(!driverStationio.GetDigital(DS_DIGIO_ARM_LOCK));
	} else {
		tHanger->HangerUpDown(0);
		tHanger->HangerLeftRight(0);
		tHanger->HangerLock(0);
	}
	// Kicker
	if (hasEnhancedIO){    //Sets the beater brush direction  ***NEEDS TESTING***
		checkDisables();


//#define NOWHEELYET true;
#ifdef NOWHEELYET
		kicker->SetPowerDesired((leftStick->GetThrottle()+1)/2);
#else
		if (driverStationio.GetDigital(DS_DIGIO_KICK_POWER_OVERRIDE)) {
			// switch is down - Use dial
			kicker->SetPowerDesired((driverStationio.GetAnalogIn(DS_ANAIN_SPARE)/3.3));
		}
		else {
			// switch is up - Use slider
			kicker->SetPowerDesired((driverStationio.GetAnalogIn(DS_ANAIN_KICK_POWER)/3.3));
		}
#endif

		float beaterSwitch = driverStationio.GetAnalogIn(DS_ANAIN_BEATER_CONTROL);
		
		if (beaterSwitch >= 3){
			kicker->TurnBeaterBrush(1);
		}
		else if (beaterSwitch <= 1){
			kicker->TurnBeaterBrush(-1);
		}
		else{
			kicker->TurnBeaterBrush(0);
		}	
	} else {  // !(hasEnhancedIO)
		kicker->SetBallSensorBroken(1);
		kicker->SetPowerLevelSensorBroken(1);
		kicker->SetPowerDesired(0.5);
		kicker->TurnBeaterBrush(0);
	}
	// Camera Servo Tilt
	if (leftStick->GetRawButton(DS_JOYBTN_CAMUP) || 
			rightStick->GetRawButton(DS_JOYBTN_CAMUP)) {
		// add 0.01 to servo per iteration
		tCamera->setServoValue(tCamera->getServoValue() + 0.01);
	} else if (leftStick->GetRawButton(DS_JOYBTN_CAMDOWN) || 
			rightStick->GetRawButton(DS_JOYBTN_CAMDOWN)) {
		tCamera->setServoValue(tCamera->getServoValue() - 0.01);
	}
#if USING_GLOW
	if (hasEnhancedIO) {
		tGlow->Process(!kicker->ShooterReady(), !driverStationio.GetDigital(DS_DIGIO_ARM_LOCK), kicker->GetRawBallSensor());
	} else {
	    tGlow->controlGlow(0);
	}
#endif
}

bool Controls::isKickerWorking()
{
	if (hasEnhancedIO)
		return !driverStationio.GetDigital(DS_DIGIO_KICK_POWER_WORKING);
	else
		return false;
}

void Controls::driverstationFeedback() {
	if (hasEnhancedIO) {			//Put all driver station feedback in here
        driverStationio.SetAnalogOut(DS_ANAOUT_KICK_POWER, (kicker->GetPowerLevel() * 2.3));
        driverStationio.SetLED(1, true);
        driverStationio.SetFixedDigitalOutput(DS_HCOUT_KICK_RDY, !(kicker->CheckPowerLevel() && kicker-> ShooterReady()));
        driverStationio.SetFixedDigitalOutput(DS_HCOUT_KICK_PREMATCH, kicker->GetPreMatchDone());
        driverStationio.SetDigitalOutput(DS_DIGIO_LED_BALL_READY, !kicker->BallAvailable());
        driverStationio.SetDigitalOutput(DS_DIGIO_LED_SPARE_GREEN, kicker->GetMagnetTouched() != 0);
	}
	
	
}

float Controls::computePower(Joystick *stick) {
	float power = stick->GetY() * stick->GetY() * stick->GetY();  //Gets value of stick cubed
	
	if (stick->GetTrigger()){ //If trigger is pulled
		//Keep at full power
	}
	else if (stick->GetRawButton(DS_JOYBTN_SLOW)){  //If slow button is pushed
		power *= .3;  //Third the power
	}
	else{  //Otherwise
		power *= .7;  //the power
	}
	
	return power;  
}

void Controls::runDebugging() {
	double curTime = GetTime();
	if (curTime < nextDebugTime)
		return;
	nextDebugTime = curTime + 0.5;

	if (leftStick->GetRawButton(DS_DBG_KICKER)) {
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "Wnt=%f Thr=%i", (double)kicker->GetPowerDesired(), (int)kicker->CheckPowerLevel());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, "CalcPtVal=%f", (double)kicker->GetMagPot());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, "RawPotVal=%f", (double)kicker->GetRawPot());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, "BallSensor=%i Broken=%i", kicker->GetRawBallSensor(), kicker->GetPowerLevelSensorBroken());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line5, "ForLimit=%i BacLimit=%i", kicker->GetForwardLimit(), kicker->GetBackwardLimit());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line6, "MT=%i GPL=%f", kicker->GetMagnetTouched(), (double)kicker->GetPowerLevel());
		driverStationLCD->UpdateLCD();
	}
	else if (leftStick->GetRawButton(DS_DBG_TDRIVE)) {
		//driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "000000000000000000000"); demo for maximum character limit on driver station
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "MotionDone=%i", drive->AutoMotionDone());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, "Angle=%f", drive->getRobotAngle());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, "LFenc=%f", drive->DistGet(0));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, "LRenc=%f", drive->DistGet(1));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line5, "RFenc=%f", drive->DistGet(2));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line6, "RRenc=%f", drive->DistGet(3));
		driverStationLCD->UpdateLCD();
	}
	else if (leftStick->GetRawButton(DS_DBG_CAM)) {
//		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "MinAngle=%f", tCamera->getMinAngle());
//		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, "MaxAngle=%f", tCamera->getMaxAngle());
		if (tCamera->searchingForVisionTarget()) {
			driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "Looking For Target");
		}
		else {
			driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "Looking For Balls");
		}
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, "TargetFound=%i", tCamera->targetFound());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, "TargetAngle=%f", tCamera->getAngleToTarget());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, "ServoValue=%f", tCamera->getServoValue());
		driverStationLCD->UpdateLCD();
	}
	else if (leftStick->GetRawButton(DS_DBG_HANGER)){
		tHanger->DebugHanger();
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "Potentiometer=%f", (double)tHanger->DebugPot());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, "Raiser=%f", (double)tHanger->DebugRaiser());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, "PID=%f", (double)tHanger->DebugPID());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, "Lock=%f", (double)tHanger->DebugLock());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, "Spd=%f   Des=%f", (double)tHanger->DebugSpeed(), (double)tHanger->DebugDesiredSpeed());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line5, "Broken=%d", (int)tHanger->IsBroken());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line6, "Lower Limit=%d", (int)tHanger->DebugLimit());
		driverStationLCD->UpdateLCD();
	}
	else if (leftStick->GetRawButton(DS_CLR_DBG)) {
		clearMessages();
	}
	//if (!leftStick->GetRawButton(DS_DBG_KICKER) && !leftStick->GetRawButton(DS_DBG_TDRIVE) && !leftStick->GetRawButton(DS_DBG_CAM)) {
	//	clearMessages();
	//}
	if (leftStick->GetRawButton(8)) {
		drive->resetGyro();
	}
}
void Controls::clearMessages() {
	driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "");
	driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, "");
	driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, "");
	driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, "");
	driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line5, "");
	driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line6, "");
	driverStationLCD->UpdateLCD();
}

float Controls::getAutoDelay() {
	float voltage = driverStation->GetAnalogIn(DS_ANAIN_AUTO_DELAY);
	return (5 * voltage) / 3.3;	
}
