#include "DashboardDataSender.h"
#include "WPILib.h"

//Unchanged from FIRST version
DashboardDataSender::DashboardDataSender(Joystick * left, Joystick * right)
{
	// these timers make sure that the data is not sent to the dashboard more
	// than 10 times per second for efficiency.
	IOTimer = new Timer();
	visionTimer = new Timer();
	IOTimer->Start();
	visionTimer->Start();
	joyLeft = left;
	joyRight = right;
	
}

DashboardDataSender::~DashboardDataSender()
{
    visionTimer->Stop();
    IOTimer->Stop();
    delete visionTimer;
    delete IOTimer;
}


//Unchanged from FIRST version
void DashboardDataSender::sendVisionData(double joyStickX,
					double gyroAngle,
					double gyroRate,
					double targetX,
					vector<Target> targets) {
	if (visionTimer->Get() < 0.1)
		return;
	visionTimer->Reset();
	Dashboard &dash = DriverStation::GetInstance()->GetHighPriorityDashboardPacker();
	dash.AddCluster(); // wire (2 elements)
	{
		dash.AddCluster(); // tracking data
		{
			dash.AddDouble(joyStickX); // Joystick X
			dash.AddDouble(((((int)gyroAngle) + 360 + 180) % 360) - 180.0); // angle
			dash.AddDouble(0.0); // angular rate
			dash.AddDouble(targetX); // other X
		}
		dash.FinalizeCluster();
		dash.AddCluster(); // target Info (2 elements)
		{
			dash.AddArray(); // targets
			{
                for (unsigned i = 0; i < targets.size(); i++) {
                    dash.AddCluster(); // targets
                    {
                        dash.AddDouble(targets[i].m_score); // target score
                        dash.AddCluster(); // Circle Description (5 elements)
                        {
                            dash.AddCluster(); // Position (2 elements)
                            {
                                dash.AddFloat((float) (targets[i].m_xPos / targets[i].m_xMax)); // X
                                dash.AddFloat((float) targets[i].m_yPos); // Y
                            }
                            dash.FinalizeCluster();

                            dash.AddDouble(targets[i].m_rotation); // Angle
                            dash.AddDouble(targets[i].m_majorRadius); // Major Radius
                            dash.AddDouble(targets[i].m_minorRadius); // Minor Radius
                            dash.AddDouble(targets[i].m_rawScore); // Raw score
                            }
                        dash.FinalizeCluster(); // Position
                        }
                    dash.FinalizeCluster(); // targets
                    }
			}
			dash.FinalizeArray();
			
			dash.AddU32((int) 0);

		}
		dash.FinalizeCluster(); // target Info
	}
	dash.FinalizeCluster(); // wire
	dash.Finalize();
}


//Modified from FIRST version
void DashboardDataSender::sendIOPortData(int robotMode, int autoMode, float autodelay) {
	if (IOTimer->Get() < 0.1)
		return;
	IOTimer->Reset();
	Dashboard &dash = DriverStation::GetInstance()->GetLowPriorityDashboardPacker();
	dash.AddCluster();
	{
		dash.AddCluster();
		{ //analog module
			for (int i = 1; i <= 8; i++) {
				//GetInstance(cRIO slot #)?
				dash.AddFloat((float) AnalogModule::GetInstance(1)->GetAverageVoltage(i));
			}
		}
		dash.FinalizeCluster();

		dash.AddCluster();
		{ //digital module
			//cRIO slot number of DIO module?
			int module = 4;
			//relay
			dash.AddU8(DigitalModule::GetInstance(module)->GetRelayForward());
			dash.AddU8(DigitalModule::GetInstance(module)->GetRelayForward());
			//Digital I/O: On/Off
			dash.AddU16((short)DigitalModule::GetInstance(module)->GetDIO());
			//Digital I/O: Input/Output
			dash.AddU16(DigitalModule::GetInstance(module)->GetDIODirection());
			dash.AddCluster();
			{ //PWMS
				for (int i = 1; i <= 10; i++) {
					dash.AddU8((unsigned char) DigitalModule::GetInstance(module)->GetPWM(i));
				}
			}
			dash.FinalizeCluster();
		}
		dash.FinalizeCluster();
		
		dash.AddCluster();
		{ //Driver station data
			//Digital inputs
			int digin = 0;
			for (int i = 1; i <= 8; i++) {
				int j = (int)(DriverStation::GetInstance()->GetDigitalIn(i));
				digin += (int)(j * pow(2,i-1));
			}
			dash.AddU8((char) digin);
			//Digital outputs
			int digout = 0;
			for (int i = 1; i <= 8; i++) {
				int j = (int)(DriverStation::GetInstance()->GetDigitalOut(i));
				digout += (int)(j * pow(2,i-1));
			}
			dash.AddU8((char) digout);
			//Analog inputs
			for (int i = 1; i <= 4; i++) {
				dash.AddFloat(DriverStation::GetInstance()->GetAnalogIn(i)); //provisional
			}
		}
		dash.FinalizeCluster();
		
		dash.AddCluster();
		{ // Joysticks
			dash.AddCluster();
			{ //Left
				dash.AddFloat(joyLeft->GetY());
				dash.AddFloat(joyLeft->GetX());
				dash.AddFloat(joyLeft->GetThrottle());
				//Digital values -- last 12 bits of short
				dash.AddU16(DriverStation::GetInstance()->GetStickButtons(DS_JOYNUM_LEFT));
			}
			dash.FinalizeCluster();
			dash.AddCluster();
			{ //Right
				dash.AddFloat(joyRight->GetY());
				dash.AddFloat(joyRight->GetX());
				dash.AddFloat(joyRight->GetThrottle());
				//Digital values -- last 12 bits of short
				dash.AddU16(DriverStation::GetInstance()->GetStickButtons(DS_JOYNUM_RIGHT));
			}
			dash.FinalizeCluster();
		}
		dash.FinalizeCluster();

		// Robot mode (Teleop/Autonomous, etc.)
		dash.AddU8((char) robotMode);
		//Alliance (red/blue)
		char allianceChar;
		DriverStation::Alliance alliance = DriverStation::GetInstance()->GetAlliance();
		if (alliance == DriverStation::kInvalid) 
			allianceChar = 2;
		else if (alliance == DriverStation::kRed)
			allianceChar = 0;
		else  //alliance == kBlue
			allianceChar = 1;
		dash.AddU8(allianceChar);
		//Auto mode
		dash.AddU8((char) autoMode);
		dash.AddFloat(autodelay);
	}
	dash.FinalizeCluster();
	dash.Finalize();
}
