#ifndef __DashboardDataFormat_h__
#define __DashboardDataFormat_h__

#include <vector>
#include "Target.h"
#include <math.h>
#include "WPILib.h"
#include "DSMap.h"
//#include "Target.h"

class Timer;

//Taken from FIRST 2010 Image Demo; modified to fit ThunderBot2010
class DashboardDataSender
{
public:
	/**
	 * Send data to the dashboard.
	 * This class sends two types of data to the dashboard program:
	 * 1. Data representing all the ports on the robot
	 * 2. Camera tracking data so the dashboard can annotate the video stream with
	 *    target information.
	 */
	DashboardDataSender(Joystick * left, Joystick * right);
	~DashboardDataSender();
    /**
	 * Send the vision tracking data.
	 * Sends the vision information to the dashboard so that the images will be annotated
	 * and the graphs will operate.
	 */

	void sendVisionData(double joyStickX,
					double gyroAngle,
					double gyroRate,
					double targetX,
					vector<Target> targets);
	/**
	 * Send IO port data to the dashboard.
	 * Send data representing the output of all the IO ports on the cRIO to the dashboard.
	 */
	//Also sends robot mode, alliance, and autonomous mode
	void sendIOPortData(int robotMode, int autoMode, float autodelay);
private:
	Timer *visionTimer;
	Timer *IOTimer;
	Joystick *joyLeft;
	Joystick *joyRight;
};
#endif // __DashboardDataFormat_h__
