#ifndef THUNDERCAM_H_
#define THUNDERCAM_H_
#include "Servo.h"
#include "ThunderAxisCamera.h"
#include "Vision/HSLImage.h"
//#include "PIDController.h"
//#include "Gyro.h"
#include "Target.h"
#include "DashboardDataSender.h"
#include "IOMap.h"
#define MINIMUM_SCORE 0.002
// min and max camera servo values
#define CAM_MIN_SERVO_VALUE 0.0
#define CAM_MAX_SERVO_VALUE 1.0
// Position Determination values:
// Outer target circle (outer rim of black ring) diameter in inches
#define TARGET_SIZE 33
// Height of center of target above ground, in inches
#define TARGET_HEIGHT 56.5
// Height of camera above ground, in inches
#define CAMERA_HEIGHT 29
// Difference in height between target center and camera, in inches
#define DELTA_HEIGHT (TARGET_HEIGHT - CAMERA_HEIGHT)
// Distance from camera to target center at calibration (camera view axis
// perpendicular to target center) in inches
/***** Recalculate with correctly sized target? *****/
#define CALIBRATION_DISTANCE 127.5
// Size of target in camera image at calibration in pixels
/***** Recalculate with correctly sized target? *****/
#define CALIBRATION_IMAGESIZE 91.2
// Camera's effective focal length (in pixels? apparently this is
// correct and works in the equation!)
#define FOCAL_LENGTH (CALIBRATION_DISTANCE * CALIBRATION_IMAGESIZE / TARGET_SIZE)

class ThunderCam {
public:
	ThunderCam(DashboardDataSender * dash);
	~ThunderCam();

	//true for looking for the vision target, or false will tell it 
	//to be looking for the balls
	void setTargetType(bool isVisionTarget);
	
	//returns true for a vision target, returns false for a ball target
	bool searchingForVisionTarget();
	
	// tells thundercam to process the targets in the image
	void Process();
	
	// returns true if there is a good enough target in the image
	bool targetFound();
	
	// gets the horizontal angle of the robot relative to the target in degrees
	// note: when angle is 0, robot is pointed directly at target; this does
	//       NOT mean that it is facing the target full on
	// -90:90 negative means left
	// returns over 9000 if target is outside of the image
	float getAngleToTarget();
	
	// gives the distance from the robot to the target in inches
	// will always be positive
	// returns less than -9000 if there is no target in the image
	float getDistanceToTarget();
	
	// tils the camera vertically using the servo
	// given the servo value
	// prevents the tilt angle from being too big
	void setServoValue(float value);
	
	// returns what the vertical camera servo is set to
	float getServoValue();
	
private:
	ThunderAxisCamera * camera;
	Servo * camVertical;
	DashboardDataSender * dds;
	// any gyro capability??
	bool searchingVisionTarget; // searching vision target (true) or ball?
	bool foundTarget; // have we found a target?
	Target bestTargetSoFar;
	Target alternateTarget;
	int notSeenCounter; // number of consecutive frames best target was not seen
	int timesSeen; // number times best target was seen
	int altTimesSeen; // same for alternate target
};
#endif
